require 'logger'
require 'mongoid'

require "gizmo/version"
require "gizmo/models"
require "gizmo/base_operation"
require "gizmo/find_all"
require "gizmo/find"
require "gizmo/search"
require "gizmo/make"
require "gizmo/modify"
require "gizmo/delete"
require "gizmo/aop"

module Gizmo
  @logger = Logger.new STDERR

  class << self
    attr_accessor :logger
  end

  # Find all items
  #
  # @param [Mongoid::Criteria] criteria
  #
  # @return [Gizmo::Response]
  def find_all(*args)
    context = create_context
    operation = FindAll.new(context)
    response = operation.call(*args)
    operation.set_response_headers response
    response
  end

  # Find an item
  #
  # @param [Mongoid::Criteria] criteria
  # @param [String, Moped::BSON::ObjecId] id
  #
  # @return [Gizmo::Response]
  def find(*args)
    context = create_context
    Find.new(context).call(*args)
  end

  def search(*args)
    context = create_context
    Search.new(context).call(*args)
  end

  # Make a new instance of an item
  #
  # @param [Mongoid::Criteria] criteria
  # @param [Hash] attrs
  #
  # @return [Gizmo::Response]
  def make(*args)
    context = create_context
    operation = Make.new(context)
    response = operation.call(*args)
    operation.set_response_headers response
    response
  end

  # Modify an item
  #
  # @param [Mongoid::Criteria] criteria
  # @param [String, Moped::BSON::ObjecId] id
  # @param [Hash] attrs
  #
  # @return [Gizmo::Response]
  def modify(*args)
    context = create_context
    Modify.new(context).call(*args)
  end

  # Delete an item
  #
  # @param [Mongoid::Criteria] criteria
  # @param [String, Moped::BSON::ObjecId] id
  #
  # @return [Gizmo::Response]
  def delete(*args)
    context = create_context
    Delete.new(context).call(*args)
  end

  private

  def create_context
    if defined?(ActionController) && self.kind_of?(ActionController::Base)
      Context.new(gizmo: self, format: request.format.to_sym, callback: params[:callback])
    else
      Context.new(gizmo: self, format: :json, callback: nil)
    end
  end


end

