require 'aquarium'

module Gizmo
  class ExceptionHandling
    include Aquarium::DSL

    def self.log_error(error)
      puts "ERROR:: #{error.message}: #{error.backtrace}"

      if defined?(Rails) && Rails.logger
        Rails.logger.error "#{error.message}: #{error.backtrace}"
      end
    end

    #around :calls_to => [:find_all, :find, :make, :modify, :delete], :in_types => /Controller$/ do |jp, obj, *args|
    types = [Gizmo::FindAll, Gizmo::Find, Gizmo::Make, Gizmo::Modify, Gizmo::Delete]
    around :calls_to => [:call], :in_types => types do |jp, operation, *args|
      production = ENV['RAILS_ENV'] == 'production'

      begin
        response = jp.proceed
      rescue Mongoid::Errors::DocumentNotFound => ex
        err = Error.new("Unable to locate #{args.first} with id=#{args[1]}", production ? [] : ex.backtrace.slice(0, 10))
        log_error err
        response = operation.create_response
        response.status = 404
        response.data = err
      rescue Mongoid::Errors::Validations => ex
        err = ValidationError.new("Validation failed for one or more fields", ex.document.errors)
        log_error err
        response = operation.create_response
        response.status = 422
        response.data = err
      rescue Exception => ex
        err = Error.new("[test] Internal server error occurred: #{ex.message}", production ? [] : ex.backtrace.slice(0, 10))
        log_error err
        response = operation.create_response
        response.status = 500
        response.data = err
      end
      response
    end
  end
end