module Gizmo
  class Make < BaseOperation


    # Make a single item using the provided criteria.
    #
    # @param [Mongoid::Criteria] criteria   the criteria to use for making a single item
    # @param [Hash] attrs                   the attributes used to make a new instance
    # @param [Proc] location_proc           a Proc that will be invoked with the ID of the newly
    #                                       created item, defaults to nil.
    #
    # @return [Gizmo::Response]
    def call(criteria, attrs, location_proc=nil)

      response = create_response
      item = criteria.create!(attrs)
      response.data = item

      if context.gizmo.respond_to?(:url_for)
        if location_proc.nil?
          url = context.gizmo.url_for item
        else
          url = location_proc.call(item.id)
        end

        response.header :Location, url unless url.nil?
      end
      response
    end

    def default_status
      201
    end
  end
end